function [pval, test] = conditional_inference(Data, nuisance, H0, beta_g0, cue_est, omg, G, J)
%--------------------------------------------------------------------------
% Data:     The data set
% nuisance: the cailbrated values
% H0:       null hypothesis on the validity of AP moments
% beta_g0:  the grid points in the baseline
% cue_est:  GMM-CUE estimator using both baseline and AP moments
% omg:      the variance-covariance matrix of all moments at "cue_est"
% G:        Jacobian matrix of all moments at "cue_est"
% J:        J test statistic from GMM-CUE 
%--------------------------------------------------------------------------
n  = size(Data,1);      k0 = 8;             k1 = 6;  

k  = k0 + k1;           S0 = [eye(k0) zeros(k0,k1)]; 

B  = 2500;              v  = mvnrnd(zeros(k,1), omg, B);

[g_hat, m_theta0] = eval_moment(cue_est, Data, nuisance, H0);

g_hat    = sqrt(n)*g_hat;
%--------------------------------------------------------------------------
% Calculate inverse of Omega_0(theta) and V(theta) for the conditional C.V.
%--------------------------------------------------------------------------
l_grid   = size(beta_g0,1); 

W0_theta = zeros(l_grid, k0^2);

V_theta  = zeros(l_grid, k0*k);

inv_omg  = pinv(omg);

parfor j = 1:l_grid 

    [~, m0_theta] = eval_moment0(beta_g0(j,:), Data, nuisance);

    omg0_theta    = m0_theta'*m0_theta/n - mean(m0_theta)'*mean(m0_theta);
    
    W0_theta(j,:) = reshape(pinv(omg0_theta),1,[]);
    
    [~, m_theta]  = eval_moment(beta_g0(j,:), Data, nuisance, H0);
     
    V0            = S0*( m_theta'*m_theta0/n - mean(m_theta)'*mean(m_theta0) )*inv_omg;
    
    V_theta(j,:)  = reshape(V0',1,[]);

end
%--------------------------------------------------------------------------
%                     Calculate the conditional C.V.
%--------------------------------------------------------------------------
g0_theta = moment0_grid(beta_g0, Data, nuisance);

J0_theta = (kron(ones(1,k0),g0_theta).*W0_theta.*kron(g0_theta,ones(1,k0)))*ones(k0^2,1); 

C        = J - n*min(J0_theta);

D        = eye(k) - G*pinv(G'*inv_omg *G)*G'*inv_omg ;

m_theta  = sqrt(n)*g0_theta - (V_theta*kron(eye(k0),g_hat'));

B_cv     = zeros(B,1);

parfor b = 1:B
        
    v_b        = D*v(b,:)';
        
    g0_theta_v = m_theta + V_theta*kron(eye(k0), v_b);
        
    J0_theta_b = ((kron(ones(1,k0),g0_theta_v).*W0_theta).*kron(g0_theta_v,ones(1,k0)))*ones(k0^2,1);

    B_cv(b)    = v_b'*inv_omg *v_b - min(J0_theta_b);

end

    pval  = mean((B_cv > C));
    
    CC_cv = quantile(B_cv,0.95);
    
    if C > CC_cv; test = 1; else, test = 0; end

end


function [f1, f2] = eval_moment(beta, Data, nuisance, H0)
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
delta = nuisance(1);     g_c   = nuisance(2);    

g_d   = nuisance(3);     Sig_c = nuisance(4); 

phi   = nuisance(5);     p     = nuisance(6);    
 
v     = nuisance(7);     q     = nuisance(8);
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
dc0   = Data(:,1);       dc1   = Data(:,2);       dc2   = Data(:,3);

dd0   = Data(:,4);       dd1   = Data(:,5);       dd2   = Data(:,6);

rm0   = Data(:,7);       rm1   = Data(:,8);       rm2   = Data(:,9);

zm0   = Data(:,10);      rb1   = Data(:,11);      n     = size(Data,1);
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
alpha = beta(:,4) + p/beta(:,1);                  rho   = beta(:,3); 

Sig_p = beta(:,2)*(1 - beta(:,3)^2);              gamma = beta(:,4);
%--------------------------------------------------------------------------
%                    Baseline Moment Conditions
%--------------------------------------------------------------------------
mu1     = v + 1/alpha;

mu2     = v^2 + 2*v/alpha + 2/(alpha^2);

m0      = zeros(n,8); 

m0(:,1) = dc1 - g_c + p*mu1;

m0(:,2) = dd1 - g_d + phi*p*mu1;

m0(:,3) = (dc1 - g_c).^2 - Sig_c - p*mu2;

m0(:,4) = (dd1 - g_d).^2 - phi^2*Sig_c - phi^2*p*mu2;

m0(:,5) = dc0.*dc2 - rho*dc0.*dc1 + dc0*(1-rho)*(p*mu1 - g_c);

m0(:,6) = dd0.*dd2 - rho*dd0.*dd1 + dd0*(1-rho)*(phi*p*mu1 - g_d);

m0(:,7) = dd0.*dc2 - rho*dd0.*dc1 + dd0*(1-rho)*(p*mu1 - g_c);

m0(:,8) = dc0.*dd2 - rho*dc0.*dd1 + dc0*(1-rho)*(phi*p*mu1 - g_d);
%--------------------------------------------------------------------------
%                Asset Pricing Moment Conditions
%--------------------------------------------------------------------------
del_b   = delta*exp( g_d - g_c );

h1      = alpha*( exp(v*gamma) - exp(v*(gamma-1))*(alpha-gamma)/(alpha-gamma+1) );

h2      = alpha*( exp(v*(gamma-phi))/(alpha-gamma+phi) - exp(v*(gamma-1))/(alpha-gamma+1) );
 
h3      = alpha*( (1-q)*exp(v*gamma) - exp(v*(gamma-phi))*(alpha-gamma)/(alpha-gamma+phi)...
          + q*exp(v*(gamma-1))*(alpha-gamma)/(alpha-gamma+1) )...
          - (phi-q)*(alpha-gamma)*mu1;
     
beta_p  = del_b*h2/(1-del_b*rho);

lam_p   = ( 1 - exp(v*(gamma-1))*alpha/(alpha-gamma+1) )/(1/delta - rho);

z_m_b   = log(del_b/(1-del_b));

omg1    = -log(delta) + g_c - Sig_c*(gamma - 0.5) - q*p*mu1 - (1-q)*p*h1/(alpha-gamma);

omg2    = q*p*mu2 - (q*p*mu1)^2 + (1-q)*p*(2*q*p*mu1 + (1-q)*p*h1/(alpha-gamma))*h1*Sig_p/((1-rho^2)*(alpha-gamma));
     
omg3    = phi*Sig_c*gamma + p^2*beta_p*lam_p*Sig_p - (phi^2*Sig_c + p^2*(beta_p^2)*Sig_p)/2 + p*h3/(alpha-gamma);

omg4    = phi^2*Sig_c + p^2*(beta_p^2)*Sig_p +p*(q - 2*phi*q + phi^2)*mu2 -(q-phi)^2*p^2*(mu1^2)...
          + ( p^2*h3^2/((alpha-gamma)^2) - p^2*(q-phi)^2*(mu1^2) )*Sig_p/(1-rho^2);
   
omg5    = (1-del_b*rho)*h3/(h2*(alpha-gamma));

omg6    = rho*(1-del_b*rho)*h3/(h2*(alpha-gamma));

m1      = zeros(n,6);

m1(:,1) = rb1 - omg1;

m1(:,2) = rb1.^2 - 2*rb1*omg1 + omg1^2 - omg2;

m1(:,3) = rm1 - omg3;

m1(:,4) = rm1.^2 - 2*rm1*omg3 + omg3^2 - omg4;

m1(:,5) = rm0.*rm1 - rm0.*(zm0 - z_m_b)*omg5 - rm0*omg3;

m1(:,6) = rm0.*rm2 - rm0.*(zm0 - z_m_b)*omg6 - rm0*omg3;

m       = [m0, m1 - ones(n,1)*H0];

f1      = mean(m);

f2      = m;

end


function [f1, f2] = eval_moment0(beta, Data, nuisance)
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
g_c   = nuisance(2);     g_d   = nuisance(3);     

Sig_c = nuisance(4);     phi   = nuisance(5);     

p     = nuisance(6);     v     = nuisance(7);     
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
dc0   = Data(:,1);       dc1   = Data(:,2);       dc2   = Data(:,3);

dd0   = Data(:,4);       dd1   = Data(:,5);       dd2   = Data(:,6);

n     = size(Data,1);
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
alpha = beta(:,4) + p/beta(:,1);                  rho   = beta(:,3); 
%--------------------------------------------------------------------------
%                    Baseline Moment Conditions
%--------------------------------------------------------------------------
mu1     = v + 1/alpha;

mu2     = v^2 + 2*v/alpha + 2/(alpha^2);

m0      = zeros(n,8); 

m0(:,1) = dc1 - g_c + p*mu1;

m0(:,2) = dd1 - g_d + phi*p*mu1;

m0(:,3) = (dc1 - g_c).^2 - Sig_c - p*mu2;

m0(:,4) = (dd1 - g_d).^2 - phi^2*Sig_c - phi^2*p*mu2;

m0(:,5) = dc0.*dc2 - rho*dc0.*dc1 + dc0*(1-rho)*(p*mu1 - g_c);

m0(:,6) = dd0.*dd2 - rho*dd0.*dd1 + dd0*(1-rho)*(phi*p*mu1 - g_d);

m0(:,7) = dd0.*dc2 - rho*dd0.*dc1 + dd0*(1-rho)*(p*mu1 - g_c);

m0(:,8) = dc0.*dd2 - rho*dc0.*dd1 + dc0*(1-rho)*(phi*p*mu1 - g_d);

f1      = mean(m0);

f2      = m0;

end


function f3 = moment0_grid(beta_grid, Data, nuisance)
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
g_c   = nuisance(2);     g_d   = nuisance(3);     

Sig_c = nuisance(4);     phi   = nuisance(5);     

p     = nuisance(6);     v     = nuisance(7);     
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
dc0   = Data(:,1);       dc1   = Data(:,2);       dc2   = Data(:,3);

dd0   = Data(:,4);       dd1   = Data(:,5);       dd2   = Data(:,6); 
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
alpha = beta_grid(:,4) + p./beta_grid(:,1);       rho   = beta_grid(:,3); 
%--------------------------------------------------------------------------
%                    Baseline Moment Conditions
%--------------------------------------------------------------------------
mu1     = v + 1./alpha;

mu2     = v^2 + 2*v./alpha + 2./(alpha.^2);

L4      = size(beta_grid,1);

m0      = zeros(L4,8);

m0(:,1) = mean(dc1 - g_c) + p*mu1;

m0(:,2) = mean(dd1 - g_d) + phi*p*mu1;

m0(:,3) = mean( (dc1 - g_c).^2 ) - Sig_c - p*mu2;

m0(:,4) = mean( (dd1 - g_d).^2 ) - phi^2*Sig_c - phi^2*p*mu2;

m0(:,5) = mean( dc0.*dc2 ) - mean(dc0.*dc1)*rho + mean(dc0)*(1-rho).*(p*mu1 - g_c);

m0(:,6) = mean( dd0.*dd2 ) - mean(dd0.*dd1)*rho + mean(dd0)*(1-rho).*(phi*p*mu1 - g_d);

m0(:,7) = mean( dd0.*dc2 ) - mean(dd0.*dc1)*rho + mean(dd0)*(1-rho).*(p*mu1 - g_c);

m0(:,8) = mean( dc0.*dd2 ) - mean(dc0.*dd1)*rho + mean(dc0)*(1-rho).*(phi*p*mu1 - g_d);

f3      = m0;

end