 function [est_beta, omg, pval_J, G, J] = gmm_cue(Data, nuisance, H0, beta_g)
%--------------------------------------------------------------------------
% Data:     The data set
% nuisance: the cailbrated values
% H0:       null hypothesis to be tested
% beta_g:   the (reparametrized) grid points used to find the GMM-CUE
%--------------------------------------------------------------------------
k0 = 8;  k1 = 6;  k = k0 + k1;  d_beta = 4;

l_grid   = size(beta_g,1);

gmm_val  = zeros(l_grid, 1);

parfor j = 1:l_grid

    gmm_val(j,:) = cue_criterion(beta_g(j,:),Data,nuisance,H0);

end

[J,loc]  = min(gmm_val);

est_beta = beta_g(loc,:);

pval_J   = 1 - chi2cdf(J, k - d_beta);

G        = zeros(d_beta, k);

S        = eye(d_beta);

epsilon  = 0.00001;

for k = 1:d_beta
    
    d_theta1 = est_beta + epsilon*S(k,:);
    
    d_theta2 = est_beta - epsilon*S(k,:);
    
    [~, gmm_d1] = cue_criterion(d_theta1,Data,nuisance,H0);
    
    [~, gmm_d2] = cue_criterion(d_theta2,Data,nuisance,H0);
    
    G(k,:) = (gmm_d1 - gmm_d2)/(2*epsilon);
    
end

G = G';

[~, ~, omg] = cue_criterion(est_beta,Data,nuisance,H0);

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Function used to calculate the full CUE-GMM criterion value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [cri_val, mean_g, omg] = cue_criterion(theta, Data, nuisance, H0)
%--------------------------------------------------------------------------
% theta:    the parameter (the orginal one/the reparametrized one)
% Data:     The data set
% nuisance: the cailbrated values
% H0:       null hypothesis to be tested
%--------------------------------------------------------------------------
%                    Calibrated  Nuisance parameters
%--------------------------------------------------------------------------
delta = nuisance(1);     g_c   = nuisance(2);    

g_d   = nuisance(3);     Sig_c = nuisance(4); 

phi   = nuisance(5);     p     = nuisance(6);    
 
v     = nuisance(7);     q     = nuisance(8);
%--------------------------------------------------------------------------
%                              Data
%--------------------------------------------------------------------------
dc0   = Data(:,1);       dc1   = Data(:,2);       dc2   = Data(:,3);

dd0   = Data(:,4);       dd1   = Data(:,5);       dd2   = Data(:,6);

rm0   = Data(:,7);       rm1   = Data(:,8);       rm2   = Data(:,9);

zm0   = Data(:,10);      rb1   = Data(:,11);      n     = size(Data,1);
%--------------------------------------------------------------------------
%                        Reparametrization
%--------------------------------------------------------------------------
alpha = theta(:,4) + p/theta(:,1);                rho   = theta(:,3); 

Sig_p = theta(:,2)*(1 - theta(:,3)^2);            gamma = theta(:,4);
%--------------------------------------------------------------------------
%                     Baseline Moment Conditions
%--------------------------------------------------------------------------
mu1     = v + 1/alpha;

mu2     = v^2 + 2*v/alpha + 2/(alpha^2);

m0      = zeros(n,8);

m0(:,1) = dc1 - g_c + p*mu1;

m0(:,2) = dd1 - g_d + phi*p*mu1;

m0(:,3) = (dc1 - g_c).^2 - Sig_c - p*mu2;

m0(:,4) = (dd1 - g_d).^2 - phi^2*Sig_c - phi^2*p*mu2;

m0(:,5) = dc0.*dc2 - rho*dc0.*dc1 + dc0*(1-rho)*(p*mu1 - g_c);

m0(:,6) = dd0.*dd2 - rho*dd0.*dd1 + dd0*(1-rho)*(phi*p*mu1 - g_d);

m0(:,7) = dd0.*dc2 - rho*dd0.*dc1 + dd0*(1-rho)*(p*mu1 - g_c);

m0(:,8) = dc0.*dd2 - rho*dc0.*dd1 + dc0*(1-rho)*(phi*p*mu1 - g_d);
%--------------------------------------------------------------------------
%                Asset Pricing Moment Conditions
%--------------------------------------------------------------------------
del_b   = delta*exp( g_d - g_c );

h1      = alpha*( exp(v*gamma) - exp(v*(gamma-1))*(alpha-gamma)/(alpha-gamma+1) );

h2      = alpha*( exp(v*(gamma-phi))/(alpha-gamma+phi) - exp(v*(gamma-1))/(alpha-gamma+1) );
 
h3      = alpha*( (1-q)*exp(v*gamma) - exp(v*(gamma-phi))*(alpha-gamma)/(alpha-gamma+phi)...
                + q*exp(v*(gamma-1))*(alpha-gamma)/(alpha-gamma+1) )...
                - (phi-q)*(alpha-gamma)*mu1;
     
beta_p  = del_b*h2/(1-del_b*rho);

lam_p   = ( 1 - exp(v*(gamma-1))*alpha/(alpha-gamma+1) )/(1/delta - rho);

z_m_b   = log(del_b/(1-del_b));

omg1    = -log(delta) + g_c - Sig_c*(gamma - 0.5) - q*p*mu1 - (1-q)*p*h1/(alpha-gamma);

omg2    = q*p*mu2 - (q*p*mu1)^2 + (1-q)*p*(2*q*p*mu1 + (1-q)*p*h1/(alpha-gamma))*h1*Sig_p/((1-rho^2)*(alpha-gamma));
     
omg3    = phi*Sig_c*gamma + p^2*beta_p*lam_p*Sig_p - (phi^2*Sig_c + p^2*(beta_p^2)*Sig_p)/2 + p*h3/(alpha-gamma);

omg4    = phi^2*Sig_c + p^2*(beta_p^2)*Sig_p +p*(q - 2*phi*q + phi^2)*mu2 -(q-phi)^2*p^2*(mu1^2)...
          + ( p^2*h3^2/((alpha-gamma)^2) - p^2*(q-phi)^2*(mu1^2) )*Sig_p/(1-rho^2);
   
omg5    = (1-del_b*rho)*h3/(h2*(alpha-gamma));

omg6    = rho*(1-del_b*rho)*h3/(h2*(alpha-gamma));

m1      = zeros(n,6);

m1(:,1) = rb1 - omg1;

m1(:,2) = rb1.^2 - 2*rb1*omg1 + omg1^2 - omg2;

m1(:,3) = rm1 - omg3;

m1(:,4) = rm1.^2 - 2*rm1*omg3 + omg3^2 - omg4;

m1(:,5) = rm0.*rm1 - rm0.*(zm0 - z_m_b)*omg5 - rm0*omg3;

m1(:,6) = rm0.*rm2 - rm0.*(zm0 - z_m_b)*omg6 - rm0*omg3;
%--------------------------------------------------------------------------
%                  Stack the Moment Conditions 
%--------------------------------------------------------------------------
m       = [m0, m1 - ones(n,1)*H0];

mean_g  = mean(m);
  
omg     = m'*m/n - mean(m)'*mean(m);

cri_val = n*mean_g*pinv(omg)*mean_g'; 

end
